/** @file Program01.cpp
 *     @brief 印刷完了通知をデモするサンプルプログラム。
 *
 *     @version 1.0, 01.27.2010
 * *//* ***************************************************************************************** */
/** @addtogroup StatusAPI SampleProgram
 * @{
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "StatusAPI.h"

void ReadStringInput(char* strBuffer, size_t nBufferSize)
{
	/* パラメータを検証する。 */
	if((strBuffer != NULL) &&
		(nBufferSize > 0))
	{
		size_t nStrLen;
		fgets(strBuffer, nBufferSize, stdin);
		/* 文字列がNULL-終了であることを確認する。 */
		strBuffer[nBufferSize - 1] = '\0';
		/* fgetsによって自動的に付加された '\n'文字を除去する  */
		nStrLen = strlen(strBuffer);
		if(	(nStrLen > 0) &&
			('\n' == strBuffer[nStrLen - 1]))
		{
			strBuffer[nStrLen - 1] = '\0';
		}
	}
}

int main(int argc, char **argv)
{
	/*~~~~~~~~~~~~~~~~~~~~~~~~~*/
	CStatusAPI oStsAPI; /* ステータスAPIオブジェクト */
	int nHandle = 0; /* プリンタハンドル */
	bool bIsInitialized; /* ステータスAPIが正常に初期化されたかを示すフラッグ */
	int nType; /* プリンターオープン時のタイプ */
	char strName[128]; /* ポート名/プリンター名 */
	DWORD dwPrnStatus; /* プリンターのステータス */
	int nAPIResult; /* ステータスAPI 呼出しの結果 */
	char strStringInput[64];	/* ユーザーの入力文字列 */
	/*~~~~~~~~~~~~~~~~~~~~~~~~~*/

	/* API初期化 */
	bIsInitialized = oStsAPI.Initialize();
	if (FALSE == bIsInitialized) {
		printf("Failed to open StatusAPI.\n");
		return (EXIT_FAILURE);
	}

	/* nType、strName用に、ユーザーから入力を取得する */
	do {
		printf("\nPlease enter type [TYPE_PORT(1)/TYPE_PRINTER(2)]: ");
		nType = 0;	/* タイプを初期化 */
		ReadStringInput(strStringInput, sizeof(strStringInput));	/* ユーザー入力を読み込み、メモリーに保存 */
		sscanf(strStringInput, "%d", &nType);	/* メモリーの文字列を解析し、数値を取得 */
	} while ((nType != TYPE_PORT) && (nType != TYPE_PRINTER));

	if (nType == TYPE_PORT) {
		printf("\nPlease enter port name: ");
	} else if (nType == TYPE_PRINTER) {
		printf("\nPlease enter printer name: ");
	}

	ReadStringInput(strName, sizeof(strName));

	nHandle = oStsAPI.BiOpenMonPrinter(nType, strName);

	if (nHandle <= 0) {
		printf("\nFailed to open printer status monitor.\n");
		return (EXIT_FAILURE);
	} else {
		printf("\nPrinter status monitor is started.");
	}

	printf("\nWaiting for printing completion from another application\n");

	/* 印刷完了通知を待つ */
	do {
		nAPIResult = oStsAPI.BiGetStatus(nHandle, &dwPrnStatus);
		if (nAPIResult != SUCCESS) {
			printf("\nFailed to get status.");
		}
	} while (!(dwPrnStatus & ASB_PRINT_SUCCESS));

	printf("\nPrinting completed.\n");

	/* ステータスAPI使用後、ステータスモニターを常に閉じる */
	nAPIResult = oStsAPI.BiCloseMonPrinter(nHandle);
	if (nAPIResult != SUCCESS) {
		printf("\nFailed to close printer status monitor.");
	}

	return (EXIT_SUCCESS);
}

/** @} **/

