/** @file Program05.cpp
 *     @brief BiLockPrinter/BiUnlockPrinterをデモするサンプルプログラム
 *
 *     @version 1.0, 01.28.2010
 *
 *//* ***************************************************************************************** */
/** @addtogroup StatusAPI SampleProgram
 * @{
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>
#include "Program05.h"

/* @brief ステータスAPIオブジェクト */
CStatusAPI g_oStsAPI;

void ReadStringInput(char* strBuffer, size_t nBufferSize)
{
	/* パラメータを検証する。 */
	if((strBuffer != NULL) &&
		(nBufferSize > 0))
	{
		size_t nStrLen;
		fgets(strBuffer, nBufferSize, stdin);
		/* 文字列がNULL-終了であることを確認する。 */
		strBuffer[nBufferSize - 1] = '\0';
		/* fgetsによって自動的に付加された '\n'文字を除去する  */
		nStrLen = strlen(strBuffer);
		if(	(nStrLen > 0) &&
			('\n' == strBuffer[nStrLen - 1]))
		{
			strBuffer[nStrLen - 1] = '\0';
		}
	}
}

void PrintCharacters(int nHandle)
{
	/*~~~~~~~~~~~~~~~~~~~~~~~~~~*/
	int nCounter;
	int nPrintCount = 0;
	int nAPIResult;
	DWORD dwWritelen;
	DWORD dwReadlen = 0;
	BYTE byReadBuff;
	DWORD dwTimeout = 3000;
	BOOL bNullTerminate = TRUE;
	BYTE byOption = 0;
	char strBuff[128 + 2];
	char strStringInput[64];
	/*~~~~~~~~~~~~~~~~~~~~~~~~~~*/

	/* strbuffの最後の２バイトは'\n' と '\0' 文字列用に予約する */
	printf("Input the string to be printed(maximum length is %u): ", (unsigned int)(sizeof(strBuff) - 2));
	/* メモリリークを防ぐためにscanf("%s")ではなく、fgetsを使用する */
	fgets(strBuff, sizeof(strBuff), stdin);
	/* strBuffの最後に'\n'と'\0'をセットする */
	strBuff[sizeof(strBuff) - 2] = '\n';
	strBuff[sizeof(strBuff) - 1] = '\0';

	printf("Input how many times the string be printed: ");
	ReadStringInput(strStringInput, sizeof(strStringInput));	/* ユーザー入力を読み込み、メモリーに保存 */
	sscanf(strStringInput, "%d", &nPrintCount);	/* メモリーの文字列を解析し、数値を取得 */

	dwWritelen = strlen(strBuff);

	/* プリンターをロック */
	do {
		nAPIResult = g_oStsAPI.BiLockPrinter(nHandle, 3000);
	} while (nAPIResult != SUCCESS);

	/* BiDirectIOExを使って文字列を印刷 */
	for (nCounter = 1; nCounter <= nPrintCount; nCounter++) {
		nAPIResult = g_oStsAPI.BiDirectIOEx(nHandle,
											dwWritelen,
											(LPBYTE) strBuff,
											&dwReadlen,
											&byReadBuff,
											dwTimeout,
											bNullTerminate,
											byOption);
	}

	/* プリンターをアンロック */
	nAPIResult = g_oStsAPI.BiUnlockPrinter(nHandle);
	if (nAPIResult != SUCCESS) {
		printf("Failed to unlock printer.\n");
	}
}

void *Step5(void *param)
{
	/*~~~~~~~~~~~~~~~~~~~~~~~~~*/
	int nHandle = 0; /* プリンタハンドル */
	int nType; /* プリンターオープン時のタイプ */
	int nAPIResult; /* ステータスAPI 呼出しの結果 */
	bool bIsInitialized; /* ステータスAPIが正常に初期化されたかを示すフラッグ */
	char strName[128]; /* ポート名/プリンター名 */
	char strStringInput[64];	/* ユーザーの入力文字列 */
	/*~~~~~~~~~~~~~~~~~~~~~~~~~*/

	/* API初期化 */
	bIsInitialized = g_oStsAPI.Initialize();
	if (FALSE == bIsInitialized) {
		printf("Failed to open StatusAPI.\n");
		return NULL;
	}

	/* nType、strName用に、ユーザーから入力を取得する */
	do {
		printf("Please enter type [TYPE_PORT(1)/TYPE_PRINTER(2)]: ");
		nType = 0;	/* タイプを初期化 */
		ReadStringInput(strStringInput, sizeof(strStringInput));	/* ユーザー入力を読み込み、メモリーに保存 */
		sscanf(strStringInput, "%d", &nType);	/* メモリーの文字列を解析し、数値を取得 */
	} while ((nType != TYPE_PORT) && (nType != TYPE_PRINTER));

	if (nType == TYPE_PORT) {
		printf("Please enter port name: ");
	} else if (nType == TYPE_PRINTER) {
		printf("Please enter printer name: ");
	}

	ReadStringInput(strName, sizeof(strName));

	nHandle = g_oStsAPI.BiOpenMonPrinter(nType, strName);

	if (nHandle <= 0) {
		printf("Failed to open printer status monitor.\n");
		return NULL;
	} else {
		printf("Printer status monitor is started.\n");
	}

	/* stdoutに全ての文字列が表示されていることを確認 */
	fflush(stdout);

	PrintCharacters(nHandle);

	/* ステータスAPI使用後、ステータスモニターを常に閉じる */
	nAPIResult = g_oStsAPI.BiCloseMonPrinter(nHandle);
	if (nAPIResult != SUCCESS) {
		printf("Failed to close printer status monitor.\n");
	}

	return NULL;
}

int main()
{
	pthread_t hThread;
	/** 子スレッドを生成 */
	pthread_create(&hThread, NULL, Step5, NULL);
	/** 終了する前に子スレッドのプロセスが完了するのを待つ*/
	pthread_join(hThread, NULL);
	printf("\n");
	return (EXIT_SUCCESS);
}

/** @} **/


